///////////////////////////////////////////////////////////////////////////////
//
// Note: This is source file is only meant as documentation for the standard 
//       library. The actual vcl code for this module is hand written.
//
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
//
// external functions (implemented as macros in vcl)
//

float4 CrossProduct(float4 a, float4 b);


///////////////////////////////////////////////////////////////////////////////
//
// Intrinsics (or whatever), the following functions map to the instruction on 
// the VU with the same name.
//

// this maps to the instruction with the same name
void xgkick(float4 address[]);
void xitop(float4 address[]);
void xtop(float4 address[]);

// int-float conversion funcs
float4 ftoi0(float4 param);
float4 ftoi4(float4 param);
float4 ftoi12(float4 param);
float4 ftoi15(float4 param);
float4 itof0(float4 param);
float4 itof4(float4 param);
float4 itof12(float4 param);
float4 itof15(float4 param);

// other arithmetic ops
float4 abs(float4 val);
float4 max(float4 a, float4 b);
float4 mini(float4 a, float4 b);
float4 mr32(float4 param);

// random number generation
float4 rget();
void   rinit(float seed);
float4 rnext();
void   rxor(float val);

// perform clipping judgement with param.XYZ against param.W
void clipW(float4 param);

// flag operations intrinsics.
int  fsand(const int mask);
bool fseq(const int mask);
int  fsor(const int mask);
// void fsset(const int val);
int  fmand(int mask);
bool fmeq(int mask);
int  fmor(int mask);
int  fcand(const int mask);
bool fceq(const int mask);
int  fcor(const int mask);
void fcset(const int val);
int  fcget();

// elementary function unit intrinsics, note, only usable for VU1
// the results are duplicated for the xyzw of the resulting float4
float4 eatan(float val);
float4 eatanXY(float4 val);
float4 eatanXZ(float4 val);
float4 eexp(float4 val);
float4 eleng(float4 val);
float4 ercpr(float val);
float4 erleng(float4 val);
float4 ersadd(float4 val);
float4 ersqrt(float val);
float4 esadd(float4 val);
float4 esin(float val);
float4 esqrt(float val);
float4 esum(float4 val);

// other ops
float rsqrt(float numerator, float denominator);
float sqrt(float val);


///////////////////////////////////////////////////////////////////////////////
//
// functions implemented in vcl_sml.i
//

void MatrixIdentity(float4 matrix[4]);
void MatrixSwap(float4 a[4], float4 b[4]);
void MatrixTranspose(float4 result[4], float4 src[4]);
void MatrixMultiply(float4 result[4], float4 left[4], float4 right[4]);
void LocalizeLightMatrix(float4 locallightmatrix[4], float4 matrix[4], float4 lightmatrix[4]);

float4 MatrixMultiplyVertex(float4 matrix[4], float4 vert);
float4 MatrixMultiplyVector(float4 matrix[4], float4 vert);
float4 VertexPersCorr(float4 vertex);
void   VertexPersCorrST(float4 outVert, float4 outST, float4 vert, float4 st);
float4 VertexFpToGsXYZ2(float4 vert);
float4 VertexFpToGsXYZ2Adc(float4 vert);
float  VectorDotProduct(float4 v1, float4 v2);
float4 VectorCrossProduct(float4 v1, float4 v2);
float4 VectorNormalize(float4 v);
float4 VectorNormalizeXYZ(float4 v);
float4 VertexLightAmb(float4 vertRGBA, float4 ambRGBA);
float4 VertexLightDir3(float4 vertRGBA, float4 vertNormal, float4 lightColors[3], float4 lightNormals[4]);
float4 VertexLightDir3Amb(float4 vertRGBA, float4 vertNormal, float4 lightColors[4], float4 lightNormals[4]);
// can not be implemented due to lack of support for float const params, float4 FogSetup(const float nearfogz, const float farfogz);
float4 VertexFogLinear(float4 xyzw, float4 fogParams);
void   VertexFogRemove(float4 xyzf, float4 fogParams);

void   AngleSinCos(float4 angles, float4 sincos);
